//
// Copyright (c) 2021, MindFusion LLC - Bulgaria.
//

using System;
using System.Drawing;
using System.Windows.Forms;
using MindFusion.Diagramming.Layout;

namespace MindFusion.Diagramming.WinForms.Samples.CS.Routing
{
    /// <summary>
    ///     Summary description for MainForm.
    /// </summary>
    public partial class MainForm : Form
    {
        private LayeredLayout m_layout = new LayeredLayout
        {
            GrowToFit = true,
            Orientation = Diagramming.Layout.Orientation.Vertical,
            LinkType = LayeredLayoutLinkType.Cascading,
            Anchoring = Anchoring.Keep,
            NodeDistance = 200,
            LayerDistance = 60,
            EnforceLinkFlow = true
        };

        private int m_nodeHeight = 4;
        private int m_nodeWidth = 4;

        public MainForm()
        {
            InitializeComponent();
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            Diagram.LinkRouter = new GridRouter();

            Diagram.MeasureUnit = MeasureUnit.Pixel;
            Diagram.ShadowsStyle = ShadowsStyle.None;
            Diagram.AdjustmentHandlesSize = 6;
            Diagram.RoutingOptions.TriggerRerouting = RerouteLinks.WhileCreating | RerouteLinks.WhenModified;
            Diagram.RoutingOptions.GridSize = 8;
            Diagram.RoutingOptions.NodeVicinitySize = 16;
            Diagram.RoutingOptions.EndOrientation = Orientation.Vertical;
            Diagram.RoutingOptions.StartOrientation = Orientation.Vertical;
            Diagram.RouteLinks = true;
            Diagram.LinkShape = LinkShape.Cascading;

            DiagramView.AutoScroll = true;

            UpdateSettingControls();
        }

        private void UpdateSettingControls()
        {
            TurnCostTrackBar.Value = Diagram.RoutingOptions.TurnCost;
            UpdateTurnCostLabel();

            CrossingCostTrackBar.Value = Diagram.RoutingOptions.CrossingCost;
            UpdateCrossingCostLabel();

            LengthCostTrackBar.Value = Diagram.RoutingOptions.LengthCost;
            UpdateLengthCostLabel();

            NodeVicinityCostTrackBar.Value = Diagram.RoutingOptions.NodeVicinityCost;
            UpdateNodeVicinityCostLabel();

            NodeVicinitySizeTrackBar.Value = (byte)Diagram.RoutingOptions.NodeVicinitySize;
            UpdateNodeVicinitySizeLabel();

            GridSizeTrackBar.Value = (byte)Diagram.RoutingOptions.GridSize;
            UpdateGridSizeLabel();

            LayerDistanceTrackBar.Value = (int)m_layout.LayerDistance;
            UpdateLayerDistanceLabel();

            NodeDistanceTrackBar.Value = (int)m_layout.NodeDistance;
            UpdateNodeDistanceLabel();

            RepeatingLinksPaddingTrackBar.Value = (int)m_layout.RepeatingLinksPadding;
            UpdateRepeatingLinksPaddingLabel();

            LinksPaddingTrackBar.Value = (int)m_layout.LinkPadding;
            UpdateLinksPaddingLabel();

            SwapPairIterationsTrackBar.Value = m_layout.SwapPairsIterations;
            UpdateSwapPairIterationsLabel();

            EndPointsOffsetTrackBar.Value = Diagram.RoutingOptions.EndPointsOffset;
            UpdateEndPointsOffsetLabel();

            WidthTextBox.Text = m_nodeWidth.ToString();
            HeightTextBox.Text = m_nodeHeight.ToString();

            if (m_layout.Anchoring == Anchoring.Keep)
                AnchoringKeepRadioButton.Checked = true;
            else if (m_layout.Anchoring == Anchoring.Ignore)
                AnchoringIgnoreRadioButton.Checked = true;
            else if (m_layout.Anchoring == Anchoring.Reassign)
                AnchoringReassignRadioButton.Checked = true;
            else
                throw new Exception("Unknown Anchoring setting");

            if (Diagram.MeasureUnit == MeasureUnit.Pixel)
                MeasureUnitPixelRadioButton.Checked = true;
            else if (Diagram.MeasureUnit == MeasureUnit.Millimeter)
                MeasureUnitMillimeterRadioButton.Checked = true;
            else
                throw new Exception("Unsupported MeasureUnit setting");
        }

        private void TurnCostTrackBar_Scroll(object sender, EventArgs e)
        {
            Diagram.RoutingOptions.TurnCost = (byte)TurnCostTrackBar.Value;
            UpdateTurnCostLabel();
        }

        private void UpdateTurnCostLabel()
        {
            TurnCostLabel.Text = "Turn cost: " + TurnCostTrackBar.Value;
        }

        private void CrossingCostTrackBar_Scroll(object sender, EventArgs e)
        {
            Diagram.RoutingOptions.CrossingCost = (byte)CrossingCostTrackBar.Value;
            UpdateCrossingCostLabel();
        }

        private void UpdateCrossingCostLabel()
        {
            CrossingCostLabel.Text = "Crossing cost: " + CrossingCostTrackBar.Value;
        }

        private void LengthCostTrackBar_Scroll(object sender, EventArgs e)
        {
            Diagram.RoutingOptions.LengthCost = (byte)LengthCostTrackBar.Value;
            UpdateLengthCostLabel();
        }

        private void UpdateLengthCostLabel()
        {
            LengthCostLabel.Text = "Length cost: " + LengthCostTrackBar.Value;
        }

        private void NodeVicinityCostTrackBar_Scroll(object sender, EventArgs e)
        {
            Diagram.RoutingOptions.NodeVicinityCost = (byte)NodeVicinityCostTrackBar.Value;
            UpdateNodeVicinityCostLabel();
        }

        private void UpdateNodeVicinityCostLabel()
        {
            NodeVicinityCostLabel.Text = "Node vicinity cost: " + NodeVicinityCostTrackBar.Value;
        }

        private void NodeVicinitySizeTrackBar_Scroll(object sender, EventArgs e)
        {
            Diagram.RoutingOptions.NodeVicinitySize = NodeVicinitySizeTrackBar.Value;
            UpdateNodeVicinitySizeLabel();
        }

        private void UpdateNodeVicinitySizeLabel()
        {
            NodeVicinitySizeLabel.Text = "Node vicinity size: " + NodeVicinitySizeTrackBar.Value;
        }

        private void GridSizeTrackBar_Scroll(object sender, EventArgs e)
        {
            Diagram.RoutingOptions.GridSize = GridSizeTrackBar.Value;
            UpdateGridSizeLabel();
        }

        private void UpdateGridSizeLabel()
        {
            GridSizeLabel.Text = "Grid size: " + GridSizeTrackBar.Value;
        }

        private void NodeDistanceBar_Scroll(object sender, EventArgs e)
        {
            m_layout.NodeDistance = NodeDistanceTrackBar.Value;
            UpdateNodeDistanceLabel();
        }

        private void UpdateNodeDistanceLabel()
        {
            NodeDistanceLabel.Text = "Node distance: " + NodeDistanceTrackBar.Value;
        }

        private void LayerDistanceTrackBar_Scroll(object sender, EventArgs e)
        {
            m_layout.LayerDistance = LayerDistanceTrackBar.Value;
            UpdateLayerDistanceLabel();
        }

        private void UpdateLayerDistanceLabel()
        {
            LayerDistanceLabel.Text = "Layer distance: " + LayerDistanceTrackBar.Value;
        }

        private void RepeatingLinksPaddingBar_Scroll(object sender, EventArgs e)
        {
            m_layout.RepeatingLinksPadding = RepeatingLinksPaddingTrackBar.Value;
            UpdateRepeatingLinksPaddingLabel();
        }

        private void UpdateRepeatingLinksPaddingLabel()
        {
            RepeatingLinksPaddingLabel.Text = "Repeat. Links padding: " + RepeatingLinksPaddingTrackBar.Value;
        }

        private void LinksPaddingBar_Scroll(object sender, EventArgs e)
        {
            m_layout.LinkPadding = LinksPaddingTrackBar.Value;
            UpdateLinksPaddingLabel();
        }

        private void UpdateLinksPaddingLabel()
        {
            LinksPaddingLabel.Text = "Links padding: " + LinksPaddingTrackBar.Value;
        }

        private void SwapPairIterationsTrackBar_Scroll(object sender, EventArgs e)
        {
            m_layout.SwapPairsIterations = SwapPairIterationsTrackBar.Value;
            UpdateSwapPairIterationsLabel();
        }

        private void UpdateSwapPairIterationsLabel()
        {
            SwapPairIterationsLabel.Text = "Swap pair iterations: " + SwapPairIterationsTrackBar.Value;
        }

        private void EndPointsOffsetTrackBar_Scroll(object sender, EventArgs e)
        {
            Diagram.RoutingOptions.EndPointsOffset = EndPointsOffsetTrackBar.Value;
            UpdateEndPointsOffsetLabel();
        }

        private void UpdateEndPointsOffsetLabel()
        {
            EndPointsOffsetLabel.Text = "End points offset: " + EndPointsOffsetTrackBar.Value;
        }

        private ShapeNode CreateNode(float width, float height)
        {
            var node = new ShapeNode
            {
                Bounds = new RectangleF(
                    0, 0, width, height),
                AnchorPattern = AnchorPattern.TopInBottomOut
        };
            Diagram.Nodes.Add(node);
            return node;
        }

        private void CreateLink(DiagramNode origin, DiagramNode destination)
        {
            var link = new DiagramLink(Diagram, origin, destination)
            {
                AutoRoute = true,
                Shape = LinkShape.Cascading,
                HeadShape = ArrowHeads.Triangle,
                HeadShapeSize = 10,
                AllowMoveEnd = false,
                AllowMoveStart = false,
            };
            Diagram.Links.Add(link);
        }

        private void ResizeToFitItems()
        {
            Diagram.ResizeToFitItems(50);
        }

        private void CreateTestGraph()
        {
            Diagram.ClearAll();

            var node1 = CreateNode(m_nodeWidth, m_nodeHeight);
            var node2 = CreateNode(m_nodeWidth, m_nodeHeight);
            var node3 = CreateNode(m_nodeWidth, m_nodeHeight);
            var node4 = CreateNode(m_nodeWidth, m_nodeHeight);

            CreateLink(node1, node2);
            CreateLink(node1, node2);
            CreateLink(node1, node4);
            CreateLink(node2, node3);
            CreateLink(node2, node4);
            CreateLink(node3, node4);

            m_layout.Arrange(Diagram);
            ResizeToFitItems();
            UpdateSettingControls();
        }

        private void Test1Button_Click(object sender, EventArgs e)
        {
            m_nodeWidth = 18;
            m_nodeHeight = 12;

            CreateTestGraph();
        }

        private void Test2Button_Click(object sender, EventArgs e)
        {
            m_nodeWidth = 180;
            m_nodeHeight = 120;

            CreateTestGraph();
        }

        private void CreateTestGraphButton_Click(object sender, EventArgs e)
        {
            CreateTestGraph();
        }

        private void RerouteAllLinksButton_Click(object sender, EventArgs e)
        {
            Diagram.RouteAllLinks();
            ResizeToFitItems();
        }

        private void ReArrangeAllButton_Click(object sender, EventArgs e)
        {
            m_layout.Arrange(Diagram);
            UpdateSettingControls();
        }

        private void ResetRoutingOptions_Click(object sender, EventArgs e)
        {
            ((RoutingOptions)Diagram.RoutingOptions).SetDefaultMode();
            UpdateSettingControls();
        }

        private void AnchoringCheckedChanged(object sender, EventArgs e)
        {
            if (AnchoringKeepRadioButton.Checked)
                m_layout.Anchoring = Anchoring.Keep;
            else if (AnchoringIgnoreRadioButton.Checked)
                m_layout.Anchoring = Anchoring.Ignore;
            else if (AnchoringReassignRadioButton.Checked)
                m_layout.Anchoring = Anchoring.Reassign;
            else
                throw new Exception("Unknown Anchoring setting");
            UpdateSettingControls();
        }

        private void MeasureUnitCheckedChanged(object sender, EventArgs e)
        {
            if (MeasureUnitPixelRadioButton.Checked)
                Diagram.MeasureUnit = MeasureUnit.Pixel;
            else if (MeasureUnitMillimeterRadioButton.Checked)
                Diagram.MeasureUnit = MeasureUnit.Millimeter;
            else
                throw new Exception("Unsupported MeasureUnit");
            UpdateSettingControls();
        }

        private void WidthTextBox_TextChanged(object sender, EventArgs e)
        {
            m_nodeWidth = Int32.Parse(WidthTextBox.Text);
        }

        private void HeightTextBox_TextChanged(object sender, EventArgs e)
        {
            m_nodeHeight = Int32.Parse(HeightTextBox.Text);
        }
    }
}